# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import re


class MalformedBuildManifest(Exception):
    def __init__(self, line):
        self.malformedLine = line

    def __str__(self):
        return repr(self.malformedLine)


class BuildManifest:
    def __init__(self, manifestFilePath):
        self.path = manifestFilePath
        self.packages = {}
        f = None
        try:
            f = open(self.path, 'r')
            for line in f:
                try:
                    package, version = re.split('\s+', line.strip(), 1)
                except:
                    raise MalformedBuildManifest(line)
                self.packages[package.strip('\n')] = version.strip('\n')
        finally:
            if f is not None:
                f.close()

    def compareTo(self, manifest):
        comparison = {}
        comparison['added'] = filter(
            (lambda x: not manifest.packages.has_key(x)),
            self.packages)
        comparison['removed'] = filter(
            (lambda x: not self.packages.has_key(x)),
            manifest.packages)
        comparison['updated'] = filter(
            (lambda x: manifest.packages.has_key(x) and 
                self.packages[x] != manifest.packages[x]),
            self.packages)
        comparison['no_change'] = filter(
            (lambda x: self.packages[x] == manifest.packages.get(x, '')),
            self.packages)
        return comparison
