#!/bin/bash

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

Load_script_configfile() {
    CONFIGFILE="${1}"

   if [ ! -f "${CONFIGFILE}" ]; then
        Error "Required configfile '%s' does not exist." ${CONFIGFILE}
        exit 1
    fi

    Action "Loading required configfile '%s'" ${CONFIGFILE}
    source "${CONFIGFILE}"
    Action_done
    
    if [ -f "${CONFIGFILE}.${HOSTNAME}" ]; then
        Action "Loading supplementary host configuration from '%s'" "${CONFIGFILE}.${HOSTNAME}"
        source "${CONFIGFILE}.${HOSTNAME}"
        Action_done
    fi
}

Load_project_config() {
    Catch_faults permit_errors

    PROJECT="${1}"
    CONFIG_BRANCH_URL="${2}"

    if [ -z "${PROJECT}" ]; then
        Error "Unable to load project configuration as no project was specified."
        exit 1
    fi

    if [ -z "${CONFIG_BRANCH_URL}" ]; then
        Error "Unable to load project configuration as no branch URL was specified."
        exit 1
    fi

    mkdir -p "${IBS_PROJECTCONFIG_CACHEDIR}" > /dev/null 2>&1

    if [ ! -d "${IBS_PROJECTCONFIG_CACHEDIR}" ]; then
        Error "Unable to create project configuration cache directory '%s'" "${IBS_PROJECTCONFIG_CACHEDIR}"
        exit 1
    fi

    IBS_PROJECTCONFIGDIR="${IBS_PROJECTCONFIG_CACHEDIR}/${PROJECT}"

    Info "Retrieving project configuration from '%s'" "${CONFIG_BRANCH_URL}"
    rm -rf "${IBS_PROJECTCONFIGDIR}"
    bzr co "${CONFIG_BRANCH_URL}" "${IBS_PROJECTCONFIGDIR}"

    if [ "$?" != "0" ] || [ ! -d "${IBS_PROJECTCONFIGDIR}" ]; then
        Error "Failed to fetch project configuration branch"
        exit 1
    else
        Load_script_configfile "${IBS_PROJECTCONFIGDIR}/project"
    fi

    Catch_faults disallow_errors
}

Export_project_config() {
    _PROJECT="${1}"
    _DESTINATION="${2}"

    if [ -z ${_PROJECT} ]; then
        Error "Unable to export project configuration as no project was specified."
        return 1
    fi

    if [ -z ${_DESTINATION} ]; then
        Error "Unable to export project configuration as no destination was specified."
        return 1
    fi

    if [ ! -d "${IBS_PROJECTCONFIGDIR}" ]; then
        Error "Unable to export project config: '%s' does not exist." "${_PROJECT}"
        return 1
    else
        Debug "Exporting project configuration to '%s'" "${_DESTINATION}/"
        bzr export "${_DESTINATION}" "${IBS_PROJECTCONFIGDIR}"
    fi

    bzr revno "${IBS_BASEDIR}" > "${_DESTINATION}/offspring-codebase.revno"
    bzr info "${IBS_BASEDIR}" > "${_DESTINATION}/offspring-codebase.info"
    bzr revno "${IBS_PROJECTCONFIGDIR}" > "${_DESTINATION}/offspring-project-config.revno"
    bzr info "${IBS_PROJECTCONFIGDIR}" > "${_DESTINATION}/offspring-project-config.info"

    dpkg-query -W > "${_DESTINATION}/${HOSTNAME}.manifest"
}

Load_defaults() {
    IBS_BASEDIR="${IBS_BASEDIR:-/srv}"
    IBS_BINDIR="${IBS_BINDIR:-${IBS_BASEDIR}/bin}"
    IBS_RESULTDIR="${IBS_RESULTDIR:-${IBS_BASEDIR}/builds}"
    IBS_WORKDIR="${IBS_WORKDIR:-/var/tmp/offspring}"
    IBS_CONFIGDIR="${IBS_CONFIGDIR:-${IBS_BASEDIR}/config}"
    IBS_PROJECTCONFIG_CACHEDIR="${IBS_PROJECTCONFIG_CACHEDIR:-/var/tmp/offspring-project-config}"

    IBS_BUILDTOOL="${IBS_BUILDTOOL:-mic}"

    IBS_RESULT_IMAGEDIR="${IBS_RESULT_IMAGEDIR:-images}"
    IBS_RESULT_CONFIGDIR="${IBS_RESULT_CONFIGDIR:-config}"
    IBS_RESULT_LOGDIR="${IBS_RESULT_LOGDIR:-}"
    IBS_RESULT_REPORTSDIR="${IBS_RESULT_REPORTSDIR:-reports}"
    IBS_RESULT_PROJDIR="${IBS_RESULT_PROJDIR:-project}"
    IBS_RESULT_SRCDIR="${IBS_RESULT_SRCDIR:-source}"

    IBS_SANITY_USER="${IBS_SANITY_USER:-`whoami`}"
    IBS_SANITY_HOST="${IBS_SANITY_HOST:-${HOSTNAME}}"
}

Check_sanity() {
    if [ "${IBS_SANITY_OVERRIDE}" != "enabled" ]; then
        Action "Performing sanity check"

        if [ `whoami` != "${IBS_SANITY_USER}" ]; then
            Error "You must run this script as ${IBS_SANITY_USER}!"
            exit 1
        fi

        if [ ${HOSTNAME} != "${IBS_SANITY_HOST}" ]; then
            Error "You must run this script on ${IBS_SANITY_HOST}!"
            exit 1
        fi

        if [ ! -d ${IBS_BASEDIR} ]; then
            Error "Basedir '%s' could not be found." "${IBS_BASEDIR}"
            exit 1
        fi

        Action_done
    else
        Warning "Sanity check skipped as IBS_SANITY_OVERRIDE is enabled."
    fi
}
