#!/bin/bash

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

Die() {
    set +e

    if [ "${_IBS_FATAL_ERROR}" = "true" ]; then
        return
    else
        _IBS_FATAL_ERROR="true"
    fi
    
    if [ "${IBS_TRACE_ERRORS}" = "enabled" ]; then
        {
            local FRAME=0
            echo "Traceback (most recent call first):"
            while read LINE_NUMBER FUNCTION_NAME FILE_NAME < <(caller ${FRAME})
            do
                printf "File: '%s', line %d, in %s\n" "${FILE_NAME}" "${LINE_NUMBER}" "${FUNCTION_NAME}"
                printf "\t  %s\n" $(sed -n "${LINE_NUMBER}p" "${FILE_NAME}" 2> /dev/null || echo "<...>")
                let FRAME+=1
            done | nl --starting-line-number=0 --no-renumber --body-numbering='pFile\:'
        }
    fi
    
    Error "A fatal error has ocurred. Shutting down."
    Stop_logging
    Catch_faults disable

    Error "A fatal error has occurred. See build log for details."
    Warning "Performing required cleanup... please wait"

    Cleanup

    Print "FAIL: %s build failed." "${PROJECT}"
    exit 1
}

Cleanup() {
    Info "Cleaning up..."

    if [ "${IBS_KEEP_BUILDFAILURES}" = "enabled" ] && [ "${_IBS_FATAL_ERROR}" = "true" ]; then
        Warning "Build failure cleanup disabled by IBS_KEEP_BUILDFAILURES=enabled"
    else
        if [ "${IBS_BUILDTOOL_LOADED}" ]
        then
            Action "Asking build tool to cleanup"
            Buildtool_cleanup
            Action_done
        else
            Info "Build tool is not loaded; skipping build tool cleanup"
        fi

        Action "Ensuring all chroot mounts are unmounted"
        for DIRECTORY in $(grep "${IBS_WORKDIR}/${PROJECT}/chroot/" /proc/mounts | awk '{ print $2 }')
        do
            sudo umount ${DIRECTORY} > /dev/null 2>&1 || true
        done
        Action_done
    fi

    Action "Remvoving build statefile"
    rm ${IBS_BUILDER_STATEFILE}
    Action_done
}

Catch_faults() {
    case "${1}" in
        disable)
            Debug "Disabling fault handler"
            set +o errtrace 
            set +e
            trap - HUP INT QUIT TERM EXIT ERR
            ;;

        enable)
            Action "Configuring fault handling"
            set -o errtrace
            set -e
            trap 'Die' HUP INT QUIT TERM EXIT ERR
            Action_done
            ;;

        disallow_errors)
            Info "Fault handler set to DISALLOW errors"
            trap 'Die' ERR
            set -e
            ;;

        permit_errors)
            Info "Fault handler set to PERMIT errors"
            trap - ERR
            set +e
            ;;

        *)
            Error "Unknown directive '%s' passed to Catch_faults function" "${1}"
            ;;
    esac
}
