#!/bin/bash

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

produceBuildComparison() {
    Catch_faults permit_errors

    if [ "${BCCTREPORT}" = "1" ] || [ "${BCCTREPORT}" = "enabled" ]
    then
        Action "Preparing to generate build comparison and change report"

        NEWMANIFESTDIR="${RESULTDIR_IMAGES}/${1}/"
        OLDMANIFESTDIR="${IBS_RESULTDIR}/${PROJECT}/previous/images/${1}/"
        NEWMANIFEST=`ls ${NEWMANIFESTDIR} 2> /dev/null | egrep -m1 "manifest|packages"`
        OLDMANIFEST=`ls ${OLDMANIFESTDIR} 2> /dev/null | egrep -m1 "manifest|packages"`

        if [ ! -f "${IBS_SCRIPTDIR}/compare-builds" ]
        then
            Error "Unable to find bcct tool; Skipping build comparison."
        elif [ -z "${1}" ]
        then
            Error "produceBuildComparison requires one argument (build type)."
        elif [ ! -f "${OLDMANIFESTDIR}/${OLDMANIFEST}" ]
        then
            Warning "Previous build failed or none found. Skipping build comparison."
        elif [ ! -f "${NEWMANIFESTDIR}/${NEWMANIFEST}" ]
        then
            Error "Current build's manifest not found. Skipping build comparison."
        else
            OUTFILE="`basename ${OLDMANIFEST} | sed 's/.manifest//'`_`basename ${NEWMANIFEST} | sed 's/.manifest//'`.bcct"

            mkdir -p "${RESULTDIR_REPORTS}"
            Action_done
            
            Action "Generating build comparison report"
            ${IBS_SCRIPTDIR}/compare-builds --hide=untouched "${OLDMANIFESTDIR}/${OLDMANIFEST}" "${NEWMANIFESTDIR}/${NEWMANIFEST}" > "${RESULTDIR_REPORTS}/${OUTFILE}"
            Action_done

            if [ -f "${IBS_SCRIPTDIR}/describe-changes" ]
            then
                Action "Generating build change description report"
                ${IBS_SCRIPTDIR}/compare-builds --machine-readable --show=updated "${OLDMANIFESTDIR}/${OLDMANIFEST}" "${NEWMANIFESTDIR}/${NEWMANIFEST}" > "${RESULTDIR_REPORTS}/${OUTFILE}.uomr"
                
                if [ "${IBS_BUILDTOOL}" = "mic" ]; then
                    _ARCH="lpia"
                else
                    _ARCH="${LH_ARCHITECTURE}"
                fi
                
                ${IBS_SCRIPTDIR}/describe-changes --arch="${_ARCH}" "${PROJECT}" "${RESULTDIR_REPORTS}/${OUTFILE}.uomr" > "${RESULTDIR_REPORTS}/changes.txt"
                Action_done
                rm "${RESULTDIR_REPORTS}/${OUTFILE}.uomr"
            else
                Warning "describe-changes tool not installed; skipping build change description report"
            fi
        fi
    else
        Info "Build comparison report disabled."
    fi

    Catch_faults disallow_errors
}

generateTreeGraph() {
    Catch_faults permit_errors

    if [ ! -z "`which tree`" ]; then
        Action "Generating tree graph of files in rootfs"
        tree -Fshapugnx -o "${RESULTDIR_REPORTS}/rootfs.tree.txt" "${TARGETFS}"
        Action_done
    else
        Error "E: Unable to generate tree graph of rootfs as required package 'tree' is not installed."
    fi

    Catch_faults disallow_errors
}

generateRootfsListing() {
    Catch_faults permit_errors

    Action "Generating simple list of files in rootfs"
    find "${TARGETFS}" | sed s,"${TARGETFS%/}",, > "${RESULTDIR_REPORTS}/rootfs.list.txt"
    Action_done

    Catch_faults disallow_errors
}

produceLicenseReport() {
    Catch_faults permit_errors

    if [ -f "${IBS_SCRIPTDIR}/describe-licensing" ]
    then
        Info "Scanning for manifest files to generate license reports under '${RESULTDIR_IMAGES}'"
        IFS="$(echo -e '\n\r')"
        for MANIFEST in $(find "${RESULTDIR_IMAGES}" -name "*.manifest")
        do
            Action "Generating license report from '${MANIFEST}'"
            ${IBS_SCRIPTDIR}/describe-licensing --config="${IBS_CONFIGDIR}/offspring.cfg" --output="${RESULTDIR_REPORTS}/license_report.$(basename ${MANIFEST} .manifest)" "${MANIFEST}"
            Action_done
        done
        unset IFS
    else
        Warning "describe-licensing tool not installed; skipping license report"
    fi

    Catch_faults disallow_errors
}
