# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from datetime import datetime

from storm import tz
from storm.locals import (
    create_database,
    Store
)

from offspring import config
from offspring.master.models import (
    BuildRequest,
    DailyBuildOrder,
    Project,
)

from master import *

def build_active_projects():
    """Queue up all active projects to be built."""

    database = create_database(config.master("db"))
    store = Store(database)
    activeProjects = store.find(Project, Project.is_active == True)
    for project in activeProjects:
        request = BuildRequest(project, project.priority, u"Nightly build")
        store.add(request)
    store.commit()

def process_pending_build_orders():
    """Queue up projects to be built per any pending daily build orders."""

    database = create_database(config.master("db"))
    store = Store(database)
    orders = store.find(
        DailyBuildOrder,
        DailyBuildOrder.hour == datetime.now(tz.tzlocal()).hour)
    for order in orders:
        for project in order.projects:
            request = BuildRequest(project, project.priority, u"Daily build")
            store.add(request)
            store.commit()
