# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

#XXX: This all needs to be rewritten.

import datetime
import platform
import smtplib
from string import Template

import offspring.config
from offspring.enums import ProjectBuildStates


def _mail_notification(sender, rcpt, msg):
    s = smtplib.SMTP('localhost')
    s.sendmail(sender, [rcpt], msg)
    s.quit()


def notify_crash(component, mailer=_mail_notification):
    sender = "%s <%s>" % (
        component, offspring.config.get("DEFAULT", "instance_email"))
    rcpt = offspring.config.get("DEFAULT", "notification_email")
    d = {
        "component": component,
        "crash_datetime": datetime.datetime.today(),
        "host_name": platform.node(),
        "instance_name": offspring.config.get("DEFAULT", "instance_name"),
        "rcpt": rcpt,
        "reply_to": offspring.config.get("DEFAULT", "discussion_email"),
        "sender": sender,
        }
    msg = Template("""From: ${sender}
To: ${rcpt}
Reply-To: ${reply_to}
Subject: ${component} has failed and is now unavailable
Priority: urgent
X-Priority: 1
X-IBS-Notification-Type: Component Failure
Comments: Automated failure message sent by ${instance_name}

Please be advised of the following component failure which may impact availability of services:

 ${component} on ${host_name} has crashed at ${crash_datetime}

The engineering team will restore full service as soon as possible.

Thank you,

The ${instance_name}
""").safe_substitute(d)
    mailer(sender, rcpt, msg)


def notify_problem(component, problem, mailer=_mail_notification):
    sender = "%s <%s>" % (
        component, offspring.config.get("DEFAULT", "instance_email"))
    rcpt = offspring.config.get("DEFAULT", "admin_email")
    d = {
        "component": component,
        "host_name": platform.node(),
        "instance_name": offspring.config.get("DEFAULT", "instance_name"),
        "problem": problem,
        "problem_datetime": datetime.datetime.today(),
        "rcpt": rcpt,
        "sender": sender,
        }
    msg = Template("""From: ${sender}
To: ${rcpt}
Subject: ${component} has reported a problem
Priority: urgent
X-Priority: 1
X-IBS-Notification-Type: Component Problem
Comments: Automated problem alert sent by ${instance_name}

Please be advised of the following component problems which may impact availability of services:

 ${component} on ${host_name} reported ${problem} at ${problem_datetime}

Thank you,

The ${instance_name}
""").safe_substitute(d)
    mailer(sender, rcpt, msg)


def notify_buildresult(build_result):
    project = build_result.project
    builder = build_result.builder
    
    build_date, build_id = build_result.name.rsplit('-', 1)
    build_url = "%s/%s/%s/%s" % (offspring.config.web('build_results_uri'), project.name, build_date, build_id)
    
    if build_result.result == ProjectBuildStates.FAILED:
        action = "<a href=\"mailto:%s\">Request help</a>" % offspring.config.master("discussion_email")
    else:
        action = "<a href=\"%s/images/\">Download build</a>" % build_url
    
    if build_result.started_at is not None:
        total_time = build_result.finished_at - build_result.started_at
    else:
        total_time = "Failed to start"
    
    sender = "%s <%s>" % (
        offspring.config.get("DEFAULT", "instance_name"),
        offspring.config.get("DEFAULT", "instance_email"))
    
    rcpt_addresses = [ subscription.user.email for subscription in project.subscriptions ]
    if build_result.requestor is not None:
        rcpt_addresses.append(build_result.requestor.email)
    
    for rcpt in set(rcpt_addresses):
        d = {
            "action" : action,
            "build_name" : build_result.name,
            "build_url" : build_url,
            "builder_name" : builder.name,
            "builder_type" : builder.machine_type,
            "buildlog_url" : "%s/build-log.txt" % build_url,
            "finished_at" : build_result.finished_at,
            "ibs_url" : offspring.config.web('ibs_uri'),
            "instance_name": offspring.config.get("DEFAULT", "instance_name"),
            "project_arch" : project.arch,
            "project_name" : project.name,
            "project_title" : project.title,
            "rcpt" : rcpt,
            "reply_to": offspring.config.get("DEFAULT", "discussion_email"),
            "result" : build_result.result,
            "sender" : sender,
            "started_at" : build_result.started_at,
            "total_time" : total_time,
            }
        msg = Template("""From: ${sender}
To: ${rcpt}
Reply-To: ${reply_to}
Subject: ${project_name} (${build_name}) build ${result}
X-IBS-Notification-Type: Build Result
X-IBS-Build-Result: ${result}
Comments: Automated build result notification sent by ${instance_name}
Content-Type: Text/HTML

<table style="border-style:none;-moz-border-radius-topleft: 15px ;-moz-border-radius-bottomleft: 15px;-moz-border-radius-topright: 15px;-moz-border-radius-bottomright: 15px;background-color:#bfbfbf;width:100%;padding: 15px;margin-bottom: 10px"><tr><td style="text-align:center;font-size: 1.5em;"><b>OEM Project Image Build Results</b></td></tr></table>

<table style="border-style:none;-moz-border-radius-topleft: 15px ;-moz-border-radius-bottomleft: 15px;-moz-border-radius-topright: 15px;-moz-border-radius-bottomright: 15px;background-color:#AFE466;float: right;padding: 15px;margin-right: 20px;">
<tr><td style="text-align:center;"><h3>Actions</h3></td></tr>
<tr><td><li style="padding: 4px 20px;border-bottom: 1px solid #F3F3F3;margin-left: 15px;list-style: none;"><a href="${build_url}/">Go to build</a></li></td></tr>
<tr><td><li style="padding: 4px 20px;border-bottom: 1px solid #F3F3F3;margin-left: 15px;list-style: none;"><a href="${buildlog_url}">View the logfile</a></li></td></tr>
<tr><td><li style="padding: 4px 20px;border-bottom: 1px solid #F3F3F3;margin-left: 15px;list-style: none;"><a href="${ibs_url}/projects/${project_name}">Queue another build</a></li></td></tr>
<tr><td><li style="padding: 4px 20px;border-bottom: 1px solid #F3F3F3;margin-left: 15px;list-style: none;">${action}</li></td></tr>
</table>

<b style="font-size: 1.1em">Build Synopsis</b>
<p style="font-size: 1.1em"><i>${result} building ${project_arch} ${project_title} (${project_name}) ${build_name}</i></p>

<b style="font-size: 1.1em">Details of Build</b>
<pre>
Project: ${project_name}
Build: ${build_name}
Arch: ${project_arch}
Started at: ${started_at}
Finished at: ${finished_at}
Total Time: ${total_time}
Builder: ${builder_name} (${builder_type})
Result: ${result}

----

Thank you,

The ${instance_name}
</pre>""").safe_substitute(d)
        _mail_notification(sender, rcpt, msg)
