#!/usr/bin/python

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

"""This script will publish pending releases to partners. """

# XXX: This script is becoming more and more of a hack. Need to refractor.

# TODO: Add logging of errors.
# TODO: Add automated mail out upon publishing.
# TODO: Add ability to try publishing a specific release.
# TODO: Add ability to retry releases which failed to publish.
# TODO: Write README file containing release notes, etc. when publishing.

import datetime
import os
import os.path
import shutil
import sys

from storm import tz
from storm.locals import (
    create_database,
    Store
)

from offspring import config
from offspring.master.models import Release

def publish_tree(src, dst):
    names = os.listdir(src)
    try:
        os.makedirs(dst)
    except:
        pass
    for name in names:
        srcname = os.path.join(src, name)
        dstname = os.path.join(dst, name)
        if os.path.isdir(srcname):
            publish_tree(srcname, dstname)
        else:
            if srcname.endswith('.asc'):
                # Do not publish development signatures.
                continue
            elif (srcname.endswith('sums.txt') and 
                    os.path.exists(config.publisher("signer_script"))):
                # Copy file into signer files directory.
                shutil.copyfile(
                    srcname, 
                    os.path.join(config.publisher("signer_files"), name))
                # Execute the signer script to have files signed.
                os.system(config.publisher("signer_script"))
                # Remove the file we copied into the signer directory.
                os.remove(
                    os.path.join(config.publisher("signer_files"), name))
                # Move the signed file to the destination.
                shutil.move(
                    os.path.join(
                        config.publisher("signer_results"), "%s.asc" % name
                    ), "%s.asc" % dstname)
            shutil.copyfile(srcname, dstname)

def run():
    database = create_database(config.publisher("db"))
    store = Store(database)
    for release in store.find(Release,
                              Release.status == Release.STATUS_PENDING):
        sys.stdout.write(
            " * Publishing pending release %s for project %s... " % 
            (release.name, release.build.project))
        sys.stdout.flush()
        images_directory = "%s/images" % release.build.result_directory(
            base_dir=config.publisher('development_pool'))
        project_directory = "%s/project" % release.build.result_directory(
            base_dir=config.publisher('development_pool'))
        release_directory = release.result_directory(
            base_dir=config.publisher('production_pool'))
        if os.path.exists("%s/%s" % 
                          (config.publisher('production_pool'),
                          release.build.project.name)):
            # images
            if os.path.exists(images_directory):
                try:
                    publish_tree(images_directory, release_directory)
                except Exception, e:
                    sys.stdout.write("FAIL.\n")
                    sys.stdout.flush()
                    release.status = Release.STATUS_ERROR
                    store.commit()
                    raise e
            else:
                sys.stdout.write("FAIL (Build not found).\n")
                sys.stdout.flush()
                release.status = Release.STATUS_ERROR
                store.commit()
            # project
            if os.path.exists(project_directory):
                try:
                    publish_tree(project_directory, release_directory)
                except Exception, e:
                    sys.stdout.write("FAIL.\n")
                    sys.stdout.flush()
                    release.status = Release.STATUS_ERROR
                    store.commit()
                    raise e
            else:
                # Don't fail if not found.
                pass
            release.status = Release.STATUS_PUBLISHED
            release.published_at = datetime.datetime.now(tz.tzlocal())
            store.commit()
            sys.stdout.write("DONE.\n")
            sys.stdout.flush()
        else:
            sys.stdout.write("SKIP.\n")
            sys.stdout.flush()
