#!/usr/bin/python

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os

from launchpadlib.credentials import Credentials
from launchpadlib.launchpad import Launchpad
from launchpadlib.uris import LPNET_SERVICE_ROOT
from storm.locals import create_database, Store

from offspring import config
from offspring.master.models import (
    LaunchpadProject, 
    LaunchpadProjectMilestone, 
    Release
)

def run():
    # Setup connection to launchpad
    cachedir = os.path.expanduser("~/.launchpadlib/cache/")
    credentials = Credentials()
    try:
        credentials.load(open(config.master("launchpad_oauth")))
        launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT)
    except:
        launchpad = Launchpad.get_token_and_login(
            'Offspring Image Build System', LPNET_SERVICE_ROOT, cachedir)
        launchpad.credentials.save(
            file(config.master("launchpad_oauth"), "w"))

    # Setup connection to database
    database = create_database(config.master("db"))
    store = Store(database)

    for launchpad_project in store.find(LaunchpadProject):
        # Get launchpad instance of the project
        try:
            p = launchpad.projects[launchpad_project.name]
        except:
            continue

        # Update our copy of the launchpad project title
        launchpad_project.title = unicode(p.title)
        store.commit()

        # Get all the milestones and save to local variable for performance.
        all_milestones = p.all_milestones

        # Create new and update existing milestones.
        for m in all_milestones:
            result = store.find(LaunchpadProjectMilestone,
                LaunchpadProjectMilestone.launchpad_project == launchpad_project,
                LaunchpadProjectMilestone.name == unicode(m.name))
            if result.is_empty():
                milestone = LaunchpadProjectMilestone(
                    m.name, m.code_name, launchpad_project, m.date_targeted, 
                    m.is_active)
                store.add(milestone)
                store.commit()
            else:
                milestone = result.one()
                milestone.date_targeted = m.date_targeted
                milestone.active = m.is_active
                if m.code_name is None:
                    milestone.title = u""
                else:
                    milestone.title = unicode(m.code_name)
                store.commit()

        # Delete milestones that no longer exist.
        rescindedMilestones = set(store.find(LaunchpadProjectMilestone,
                    LaunchpadProjectMilestone.launchpad_project == launchpad_project
                    ).values(LaunchpadProjectMilestone.name)
                ).difference(set([m.name for m in all_milestones]))
        for rescindedMilestoneName in rescindedMilestones:
            result = store.find(
                LaunchpadProjectMilestone,
                LaunchpadProjectMilestone.launchpad_project == launchpad_project,
                LaunchpadProjectMilestone.name == rescindedMilestoneName)
            # Severe any references to milestone from releases.
            store.find(
                Release, Release.milestone_id.is_in(
                    result.values(LaunchpadProjectMilestone.id)) 
                ).set(milestone_id=None)
            result.remove()
            store.commit()
