# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from base64 import decodestring
import httplib
from SimpleXMLRPCServer import SimpleXMLRPCRequestHandler
import xmlrpclib

from offspring import config

### Slave Components 

class BasicAuthXMLRPCRequestHandler(SimpleXMLRPCRequestHandler):
    """XMLRPC Request Handler that supports basic digest authentication."""

    def do_POST(self):
        """Override do_POST method to perform http digest authentication."""
        if self.headers.has_key("authorization"):
            auth = self.headers['authorization']
            auth = auth.replace("Basic ", "")
            decoded_auth = decodestring(auth)
            if decoded_auth == config.slave("xmlrpc_server_auth"):
                return SimpleXMLRPCRequestHandler.do_POST(self)
        # Authorization failed or was not attempted.
        self.send_response(401)
        self.end_headers()
        return

### Master Components

class TimeoutHTTPConnection(httplib.HTTPConnection):
    def connect(self):
        """Override the standard connect() methods to set a timeout"""
        ret = httplib.HTTPConnection.connect(self)
        self.sock.settimeout(int(config.master("slave_connect_timeout")))
        return ret

class TimeoutHTTP(httplib.HTTP):
    _connection_class = TimeoutHTTPConnection

class TimeoutTransport(xmlrpclib.Transport):
    """XMLRPC Transport to setup a socket with defined timeout"""

    def make_connection(self, host):
        host, extra_headers, x509 = self.get_host_info(host)
        return TimeoutHTTP(host)
