# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import datetime
import threading
import subprocess


class MonitoredThread(threading.Thread):
    STATE_ACTIVE, STATE_FAILED, STATE_SUCCESS = (
        u"ACTIVE", 
        u"FAILED", 
        u"COMPLETED"
    )

    def __init__(self, master=None, arguments=[]):
        self.master = master
        self.arguments = arguments
        self.state = None
        self.startedAt = None
        self.finishedAt = None
        self.errorCode = None
        threading.Thread.__init__(self)
        self.daemon = True
        self.setup()

    def setup(self):
        pass

    def getState(self):
        return self.state

    def getErrorCode(self):
        return self.errorCode

    def getStartedAt(self):
        return self.startedAt

    def getFinishedAt(self):
        return self.finsedAt

    def getArguments(self):
        return self.arguments

    def preExecution(self):
        pass

    def execute(self):
        raise NotImplementedError

    def postExecution(self):
        pass

    def run(self):
        self.preExecution()
        self.state = MonitoredThread.STATE_ACTIVE
        self.startedAt = datetime.datetime.today()
        self.errorCode = self.execute()
        self.finishedAt = datetime.datetime.today()
        if self.errorCode == 0:
            self.state = MonitoredThread.STATE_SUCCESS
        else:
            self.state = MonitoredThread.STATE_FAILED
        self.postExecution()


class MonitoredProcess(MonitoredThread):
    def __init__(self, master=None, arguments=[], shell=False,
                 stdout=subprocess.PIPE, stderr=subprocess.PIPE):
        self.process = None
        self.shell = shell
        self.stdout = stdout
        self.stderr = stderr
        MonitoredThread.__init__(self, master, arguments)

    def execute(self):
        self.process = subprocess.Popen(
            [self.getCommandPath()] + self.arguments,
            shell=self.shell,
            stdout=self.stdout,
            stderr=self.stderr)
        self.stdoutdata, self.stderrdata = self.process.communicate()
        return self.process.returncode

    def getCommandPath(self):
        raise NotImplementedError
