# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.db import models
from django.forms import ModelChoiceField, ModelForm, Textarea, TextInput
from django.forms import fields

from offspring.web.queuemanager.models import (
    Project,
    Release,
    LaunchpadProject
)

from offspring.web.queuemanager.widgets import SelectWithAddNew

class ProjectBaseForm(ModelForm):
    status = fields.CharField(max_length=200,
        widget=fields.Select(choices=Project.STATUS_CHOICES), required=True)

    class Meta:
        model = Project
        widgets = {
            'name' : TextInput(attrs={'style': 'text-transform: lowercase;'}),
            'series' : TextInput(attrs={'style': 'text-transform: lowercase;'}),
            'config_url': TextInput(attrs={'size': 50}),
            'notes' : Textarea(attrs={'cols': 73, 'rows' : 4}),
        }

    def clean_name(self):
        return self.cleaned_data['name'].lower()

    def clean_series(self):
        return self.cleaned_data['series'].lower()


class CreateProjectForm(ProjectBaseForm):
    launchpad_project = ModelChoiceField(
        LaunchpadProject.objects, widget=SelectWithAddNew, required=False)
    class Meta(ProjectBaseForm.Meta):
        exclude = ('priority', 'is_active', 'suite')


class EditProjectForm(ProjectBaseForm):
    launchpad_project = ModelChoiceField(
        LaunchpadProject.objects, widget=SelectWithAddNew, required=False)
    class Meta(ProjectBaseForm.Meta):
        exclude = ('name', 'priority', 'is_active')


class ReleaseForm(ModelForm):
    class Meta:
        model = Release
        fields = ( 'name', 'milestone', 'tag', 'checklist_url', 'notes')


class LaunchpadProjectForm(ModelForm):
    class Meta:
        model = LaunchpadProject
        widgets = {
            'name' : TextInput(attrs={'style': 'text-transform: lowercase;'}),
        }

    def clean_name(self):
        return self.cleaned_data['name'].lower()
