# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import os

from django.conf import settings
from django.conf.urls.defaults import (
    handler404,
    handler500,
    include,
    patterns,
    url
)
from django.contrib import admin

from piston.resource import Resource

import offspring.web
from offspring.web.queuemanager.models import Lexbuilder
from offspring.web.queuemanager.handlers import (
    BuildManifestHandler,
    BuildManifestComparisonHandler,
    BuildResultHandler,
    BuildRequestHandler,
    LaunchpadProjectMilestoneHandler,
    ProjectNotificationSubscriptionHandler,
    ReleaseHandler
)
from offspring.web.queuemanager.views import (
    project_create,
    project_edit,
    secure_object_list,
    secure_object_detail
)

admin.autodiscover()

milestone_handler = Resource(LaunchpadProjectMilestoneHandler)
buildrequest_handler = Resource(BuildRequestHandler)
buildresult_handler = Resource(BuildResultHandler)
projectNotificationSubscription_handler = Resource(
    ProjectNotificationSubscriptionHandler)
manifest_handler = Resource(BuildManifestHandler)
manifestComparison_handler = Resource(BuildManifestComparisonHandler)
release_handler = Resource(ReleaseHandler)

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'^openid/', include('django_openid_auth.urls')),
    (r'^logout$', 'django.contrib.auth.views.logout'),
    (r'^builders/$', secure_object_list, {
        'queryset' : Lexbuilder.objects.order_by("-is_active", "-machine_type", "-created_at"),
        'template_name' : 'queuemanager/builders.html',
        'template_object_name' : 'builder',
        'extra_context' : { 'pillar' : 'builders', },
    }, 'builders_list'),
    (r'^builders/(?P<slug>[^/]+)/$', secure_object_detail, {
        'queryset' : Lexbuilder.objects.all(),
        'slug_field' : 'name',
        'template_name' : 'queuemanager/builder_details.html',
        'template_object_name' : 'builder',
        'extra_context' : { 'pillar' : 'builders', },
    }, 'builder_details'),
    (r'^builders/\+api/(?P<builderName>[^/]+)/builds/$', buildresult_handler),
    (r'^builds/\+api/$', buildresult_handler),
    (r'^builds/\+api/(?P<projectName>[^/]+)/$', buildresult_handler),
    (r'^builds/\+api/(?P<projectName>[^/]+)/(?P<buildName>[^/]+)/$', buildresult_handler),
    (r'^builds/\+api/(?P<projectName>[^/]+)/(?P<buildName>[^/]+)/manifest/\+comparison', manifestComparison_handler),
    (r'^builds/\+api/(?P<projectName>[^/]+)/(?P<buildName>[^/]+)/manifest/$', manifest_handler),
    (r'^builds/(?P<projectName>[^/]+)/(?P<buildName>[^/]+)/\+publish$', 'offspring.web.queuemanager.views.build_publish'),
    (r'^builds/(?P<projectName>[^/]+)/(?P<buildName>[^/]+)/$', 'offspring.web.queuemanager.views.build_details'),
    (r'^builds/(?P<projectName>[^/]+)/$', 'offspring.web.queuemanager.views.builds'),
    (r'^builds/$', 'offspring.web.queuemanager.views.builds'),
    (r'^launchpad_project/\+add/$', 'offspring.web.queuemanager.views.launchpad_project_create'),
    (r'^project-groups/(?P<projectGroupName>[^/]+)/$', 'offspring.web.queuemanager.views.projectgroup_details'),
    url(r'^projects/\+add/$', project_create, name="project_create"),
    url(r'^projects/(?P<projectName>[^/]+)/\+edit$', project_edit, name="project_edit"),
    (r'^projects/(?P<projectName>[^/]+)/sources\.list$', 'offspring.web.queuemanager.views.project_development_sourcesList'),
    (r'^projects/(?P<projectName>[^/]+)/\+api/subscription/$', projectNotificationSubscription_handler),
    (r'^projects/(?P<projectName>[^/]+)/\+api/releases/$', release_handler),
    (r'^projects/(?P<projectName>[^/]+)/\+api/buildrequest/$', buildrequest_handler),
    (r'^projects/(?P<projectName>[^/]+)/\+api/buildrequest/(?P<request_id>[^/]+)/$', buildrequest_handler),
    (r'^projects/(?P<projectName>[^/]+)/\+build$', 'offspring.web.queuemanager.views.queue_build'),
    (r'^projects/(?P<projectName>[^/]+)/\+builds$', 'offspring.web.queuemanager.views.builds'),
    (r'^projects/(?P<projectName>[^/]+)/$', 'offspring.web.queuemanager.views.project_details'),
    (r'^projects/$', 'offspring.web.queuemanager.views.projects'),
    (r'^schedule/\+api/milestones/$', milestone_handler),
    (r'^schedule/\+(?P<milestoneType>[^/]+)-milestones$', 'offspring.web.queuemanager.views.milestones'),
    (r'^schedule/$', 'offspring.web.queuemanager.views.schedule'),
    (r'^$', 'offspring.web.queuemanager.views.projects'),
)

if settings.DEBUG:
    from django.views.decorators.cache import cache_page
    from django.views import static

    urlpatterns += patterns('',
        (r'^assets/(?P<path>.*)$', cache_page(static.serve, 60 * 60 * 24), {
            'document_root': os.path.join(offspring.web.__path__[0], 'media')
            }
        ),
    )
